use super::{data::LogicData, math::LogicLong};
use crate::{
    allocator,
    ffi_util::import,
    sc_string::ScString,
    util::data_stream::{ByteStream, ChecksumEncoder},
};

#[repr(C)]
pub struct LogicAvatarVTable {
    finalizer_1: extern "C" fn(*mut LogicClientAvatar),
    finalizer_2: extern "C" fn(*mut LogicClientAvatar),
    pub destruct: extern "C" fn(*mut LogicClientAvatar),
    pub encode: extern "C" fn(*const LogicClientAvatar, *mut ChecksumEncoder),
    pub decode: extern "C" fn(*mut LogicClientAvatar, *mut ByteStream),
    set_logic_game_mode: extern "C" fn(*mut LogicClientAvatar, usize),
    is_npc_avatar: extern "C" fn(*const LogicClientAvatar) -> bool,
    get_checksum: extern "C" fn(*const LogicClientAvatar) -> i32,
}

#[repr(C)]
pub struct LogicClientAvatar {
    pub vtable: &'static LogicAvatarVTable,
    pub logic_game_mode: usize,
    pub id: LogicLong,         // 8
    pub account_id: LogicLong, // 16
    pub home_id: LogicLong,    // 24
    unk_gap_1: [u8; 28],       // 32 - 60
    pub name: ScString,        // 60
    pub score: i32,            // 80
    unk_gap_2: [u8; 28],       // 84 - 112
    pub level: i32,            // 112
    unk_gap_3: [u8; 68],       // 116 - 184
    pub npc_win_count: i32,    // 184
    pub npc_lose_count: i32,   // 188,
    unk_192: bool,
    pub name_set_by_user: bool,
    unk_194: bool,
    pub name_change_state: i32,    // 196
    pub arena: &'static LogicData, // 200
}

impl LogicClientAvatar {
    pub fn new() -> &'static mut LogicClientAvatar {
        import!(logic_client_avatar_ctor(ptr: *mut LogicClientAvatar) -> () = 0xEAE00 + 1);
        let instance = allocator::new::<Self>();
        logic_client_avatar_ctor(instance);
        unsafe { &mut *instance }
    }

    pub fn clone(&self) -> &'static mut LogicClientAvatar {
        let mut stream = ByteStream::new(128);

        (self.vtable.encode)(self, &mut stream.parent);
        stream.reset_offset();

        let cloned_avatar = LogicClientAvatar::new();
        (cloned_avatar.vtable.decode)(cloned_avatar, &mut stream);
        stream.destruct();

        cloned_avatar
    }
}
