use crate::util::data_stream::{ByteStream, ChecksumEncoder};

use super::{json::LogicJSONNode, math::LogicLong, mode::LogicGameMode};

mod manager;
pub use manager::LogicCommandManager;

#[repr(C)]
pub struct LogicCommandVTable {
    pub destruct: extern "C" fn(ptr: *mut LogicCommand),
    finalizer_1: extern "C" fn(ptr: *mut LogicCommand),
    finalizer_2: extern "C" fn(ptr: *mut LogicCommand),
    pub execute: extern "C" fn(ptr: *const LogicCommand, logic_game_mode: *mut LogicGameMode),
    pub get_command_type: extern "C" fn() -> i16,
    pub encode: extern "C" fn(ptr: *const LogicCommand, encoder: *mut ChecksumEncoder),
    pub decode: extern "C" fn(ptr: *mut LogicCommand, stream: *mut ByteStream),
    pub get_command_gold_cost:
        extern "C" fn(ptr: *const LogicCommand, logic_game_mode: *const LogicGameMode) -> i32,
    pub get_command_diamond_cost:
        extern "C" fn(ptr: *const LogicCommand, logic_game_mode: *const LogicGameMode) -> i32,
    pub get_json_for_replay: extern "C" fn(ptr: *const LogicCommand) -> *const LogicJSONNode,
    pub load_from_json: extern "C" fn(ptr: *mut LogicCommand, json: *const LogicJSONNode),
    pub is_saved_for_replay: extern "C" fn(ptr: *const LogicCommand) -> bool,
}

#[repr(C)]
pub struct LogicCommand {
    pub vtable: &'static LogicCommandVTable,
    unk: bool,
    pub execute_tick: i32,
    pub tick_when_given: i32,
    pub executor_account_id: LogicLong,
}
