use crate::{ffi_util::import, resources::CSVNode, sc_string::ScString};

pub struct LogicDataTables;

#[repr(C)]
pub struct LogicData {
    unk_gap_1: [u8; 16],
    pub global_id: i32, // 16
}

#[repr(C)]
pub struct LogicNpcData {
    pub parent: LogicData,
    unk_gap_1: [u8; 12], // 20 - 32
    pub location: &'static LogicLocationData,
}

#[repr(C)]
pub struct LogicLocationData {
    pub parent: LogicData,
}

#[repr(C)]
pub struct LogicSpellData {
    pub parent: LogicData,
}

impl LogicLocationData {
    pub fn get_tilemap_file_name(&self) -> String {
        import!(logic_location_data_get_tilemap_file_name(data: *const LogicLocationData) -> *const ScString = 0x103EC4 + 1);
        let file_name = logic_location_data_get_tilemap_file_name(self);
        unsafe { (&*file_name).to_string() }
    }
}

impl LogicSpellData {
    pub fn get_max_level_index(&self) -> i32 {
        import!(logic_spell_data_get_max_level_index(data: *const LogicSpellData) -> i32 = 0x10A2F4 + 1);
        logic_spell_data_get_max_level_index(self)
    }
}

impl LogicDataTables {
    pub fn get_data_by_id(global_id: i32) -> Option<&'static LogicData> {
        import!(logic_data_tables_get_data_by_id(global_id: i32) -> *const LogicData = 0xFCB88 + 1);
        let data = logic_data_tables_get_data_by_id(global_id);
        (!data.is_null()).then(|| unsafe { &*data })
    }

    pub fn get_spell_count() -> usize {
        import!(logic_data_tables_get_spell_count() -> usize = 0xFE974 + 1);
        logic_data_tables_get_spell_count()
    }

    pub fn get_spell_at(index: usize) -> &'static LogicSpellData {
        import!(logic_data_tables_get_spell_at(index: usize) -> *const LogicSpellData = 0xFE98C + 1);
        unsafe { &*logic_data_tables_get_spell_at(index) }
    }

    pub fn init_tilemap(csv: CSVNode, id: i32) {
        import!(logic_data_tables_init_tilemap(csv: *const u8, id: i32) -> () = 0xFE49C + 1);
        logic_data_tables_init_tilemap(csv.0, id);
    }
}
