use crate::{
    allocator,
    ffi_util::{import, Nullable},
    util::data_stream::{ByteStream, ChecksumEncoder},
};

use super::{
    data::LogicSpellData,
    math::LogicLong,
    spell::{LogicSpellCollection, LogicSpellDeck},
};

#[repr(C)]
pub struct LogicClientHome {
    pub change_listener: &'static LogicHomeChangeListener,
    pub logic_game_mode: usize,
    pub id: Nullable<LogicLong>,                // 8
    pub spell_deck: LogicSpellDeck,             // 12
    pub spell_collection: LogicSpellCollection, // 44
    unk_gap_1: [u8; 332],                       // 60 - 392
}

#[repr(C)]
pub struct LogicHomeChangeListener {
    pub _vtable: usize,
}

impl LogicClientHome {
    pub fn new() -> &'static mut Self {
        import!(logic_client_home_ctor(ptr: *mut LogicClientHome) -> () = 0x114D40 + 1);
        let instance = allocator::new::<Self>();
        logic_client_home_ctor(instance);
        unsafe { &mut *instance }
    }

    pub fn has_spell(&self, spell_data: &LogicSpellData) -> bool {
        import!(logic_client_home_has_spell(ptr: *const LogicClientHome, spell_data: *const LogicSpellData) -> bool = 0x116EDC + 1);
        logic_client_home_has_spell(self, spell_data)
    }

    pub fn encode(&self, encoder: &mut ChecksumEncoder) {
        import!(logic_client_home_encode(ptr: *const LogicClientHome, encoder: *mut ChecksumEncoder) -> () = 0x115CD4 + 1);
        logic_client_home_encode(self, encoder);
    }

    pub fn decode(&mut self, stream: &mut ByteStream) {
        import!(logic_client_home_decode(ptr: *mut LogicClientHome, stream: *mut ByteStream) -> () = 0x115FD0 + 1);
        logic_client_home_decode(self, stream);
    }

    pub fn destruct(&self) {
        import!(logic_client_home_destruct(ptr: *const LogicClientHome) -> () = 0x115058 + 1);
        logic_client_home_destruct(self);
    }

    pub fn clone(&self) -> &'static mut LogicClientHome {
        let mut stream = ByteStream::new(128);

        self.encode(&mut stream.parent);
        stream.reset_offset();

        let cloned_home = LogicClientHome::new();
        cloned_home.decode(&mut stream);
        stream.destruct();

        cloned_home
    }
}
