use crate::{allocator::malloc, ffi_util::import};

use super::{message::PiranhaMessage, Connection, LogicScrollMessageFactory, RC4Encrypter};

#[repr(transparent)]
pub struct Messaging(pub *const u8);

impl Messaging {
    pub fn new(queue_size: usize) -> Self {
        import!(messaging_ctor(ptr: *const u8, queue_size: i32) -> () = 0x138C2C + 1);
        let instance = malloc(300);
        messaging_ctor(instance, queue_size as i32);
        Self(instance)
    }

    pub fn set_encrypters(
        &mut self,
        encrypter: *const RC4Encrypter,
        decrypter: *const RC4Encrypter,
    ) {
        import!(messaging_set_encrypters(ptr: *const u8, en: *const RC4Encrypter, de: *const RC4Encrypter, a4: bool) -> () = 0x138BFC + 1);
        messaging_set_encrypters(self.0, encrypter, decrypter, false);
    }

    pub fn on_receive(&self, connection: &Connection) {
        import!(messaging_on_receive(ptr: *const u8, connection: *const Connection) -> () = 0x139650 + 1);
        messaging_on_receive(self.0, connection);
    }

    pub fn next_message(&self) -> Option<&'static PiranhaMessage> {
        import!(messaging_next_message(ptr: *const u8) -> *const PiranhaMessage = 0x1390B4 + 1);
        let message = messaging_next_message(self.0);
        (!message.is_null()).then(|| unsafe { &*message })
    }

    pub fn send<T>(&self, message: *const T) {
        import!(messaging_send(ptr: *const u8, message: usize) -> () = 0x138FB8 + 1);
        messaging_send(self.0, message as usize);
    }

    pub fn on_wakeup(&self, connection: &Connection) {
        import!(messaging_on_wakeup(ptr: *const u8, connection: *const Connection) -> () = 0x1394C4 + 1);
        messaging_on_wakeup(self.0, connection);
    }

    pub fn get_connection(&self) -> &Connection {
        unsafe { &*(self.0.wrapping_add(64) as *const Connection) }
    }

    pub fn get_connection_mut(&self) -> &mut Connection {
        unsafe { &mut *(self.0.wrapping_add(64) as *mut Connection) }
    }

    pub fn set_message_factory(&mut self, factory: &'static LogicScrollMessageFactory) {
        unsafe { *(self.0.wrapping_add(4) as *mut *const LogicScrollMessageFactory) = factory }
    }
}
