use std::{
    ffi::{CStr, CString},
    fmt,
};

use crate::{allocator, ffi_util::import};

#[repr(C)]
pub struct ScString {
    unk_1: i32,
    length: i32,
    content: [u8; 8],
    unk_2: i32,
}

impl<S> From<S> for &'static ScString
where
    S: AsRef<str>,
{
    fn from(value: S) -> Self {
        import!(string_ctor(ptr: *const ScString, data: *const u8) -> () = 0x13AF4C + 1);
        let sc_string = unsafe { &*allocator::new::<ScString>() };

        string_ctor(sc_string, CString::new(value.as_ref()).unwrap().as_ptr());
        sc_string
    }
}

impl ScString {
    pub fn new<S: AsRef<str>>(value: S) -> &'static Self {
        From::from(value)
    }

    pub fn assign(&mut self, content: &str) {
        import!(string_assignment_operator(lhs: *mut ScString, rhs: *const ScString) -> () = 0x13B16C + 1);
        string_assignment_operator(self, ScString::new(content));
    }
}

impl fmt::Display for ScString {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        let length = self.length as usize;
        if length + 1 > 8 {
            let c_string = usize::from_le_bytes(self.content[..4].try_into().unwrap()) as *const u8;
            unsafe {
                write!(
                    f,
                    "{}",
                    CStr::from_ptr(c_string).to_string_lossy().to_string()
                )
            }
        } else if length > 0 {
            unsafe {
                write!(
                    f,
                    "{}",
                    CStr::from_ptr(self.content.as_ptr())
                        .to_string_lossy()
                        .to_string()
                )
            }
        } else {
            Ok(())
        }
    }
}

impl fmt::Debug for ScString {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "\"{}\"", self)
    }
}
